//imports, consts, and globals
#include <Adafruit_NeoPixel.h>
#include "pitches.h"
#include "songs.h"
const int SWITCH_PIN = 2;
#define PIN 9
long lightTimer;
long flowerTimer;
int lightInc;
int flowerInc;
int r = 0;
int g = 0;
int b = 0;
const int SPEAKER_PIN = 8;
bool hasStarted = false;

Adafruit_NeoPixel strip = Adafruit_NeoPixel(1, PIN, NEO_GRB + NEO_KHZ800);

void setup() {
  //initiate serial output, pins, Adafruit "strip"
  Serial.begin(9600);
  pinMode(SWITCH_PIN, INPUT_PULLUP);
  pinMode(SPEAKER_PIN, OUTPUT);
  strip.begin();
  strip.show();
}

void loop() {
  //read the switch to check if a flower is on the LED in the pot
  int value = digitalRead(SWITCH_PIN);
  if (value == 1) {
    if (!hasStarted){ //if the flower is in the pot and music isn't playing, start making noise and changing color
      playMusic(startup,startupSize, startupDurations, false);
      hasStarted = true;
    }
    delay(250); //short delay after playing tone and before playing music
    int temp = random(2,10); //random song selections
    //if (temp == 0){playMusic(acTown,acTownSize, acTownDurations, true);}//AC Town theme
    //else if (temp == 1){playMusic(zelda,zeldaSize, zeldaDurations, true);}//Zelda treasure tune
    if (temp == 2){playMusic(mario,marioSize, marioDurations, true);}//Mario main theme
    else if (temp == 3){playMusic(underworld,underworldSize, underworldDurations, true);}//Mario underground theme
    else if (temp == 4){playMusic(mp,mpSize, mpDurations, true);} //Mukkathe Penne
    else if (temp == 5){playMusic(jeopardy,jeopardySize, jeopardyDurations, true);} //Jeopardy theme
    else if (temp == 6){playMusic(black,blackSize, blackDurations, true);} //Paint it Black by the Rolling Stones
    else if (temp == 7){playMusic(joy, joySize, joyDurations, true);} //Ode to Joe samples 1 and 2
    else if (temp == 8){playMusic(undertale,undertaleSize, undertaleDurations, true);} //Undertale theme
    else if (temp == 9){playMusic(ode2,ode2Size, ode2Durations, true);} //Ode to Joe sample 3
    value = digitalRead(SWITCH_PIN);
    if (value == 0){//if the flower is removed, play shutdown tones and stop music/colors
      playMusic(endTone,endToneSize, endToneDurations, false);
      displayColor(0,0,0);
      hasStarted = false;
      }
    }
}

void displayColor(int red, int green, int blue) { //wrapper function for Adafruit LED color display
  strip.setPixelColor(0, strip.Color(green, red, blue)); //GRB color input
  strip.show(); //display inputted color
  }

void playMusic(int melody[], int size, int durations[], bool interruptable) {
  //iterate though the int array of notes until note count is equal to or greater than size
  for (int thisNote = 0; thisNote < size; thisNote++) {
    //startup and shutdown will always play through in their entirety
    //other songs can be interrupted after each note is done playing by removing the flower
    if (interruptable and digitalRead(SWITCH_PIN)==0){ //if interruptable and flower removed, stop the song
      break;
      }
    // to calculate the note duration, take one second divided by the note type.
    //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
    int noteDuration = 1000 / durations[thisNote];
    if (interruptable){
      //flower tempo varies by color
      //blue is 25% slower, green is normal speed, red is 25% faster
      noteDuration = noteDuration * 1.0; //1.25 -> slow, 1.0 -> normal, .75 -> fast
      displayColor(0,0,0);
      }
    if (melody[thisNote] != 0){ //if note in note array is not a rest
      tone(SPEAKER_PIN, melody[thisNote], noteDuration); 
      //play the note and change to a random shade of specifified color
      /*Blue LED random integer ranges
      r = random(0,50);
      g = random(0,256);
      b = random(200,256);
      */
      //Green LED random integer ranges
      r = random(0,100);
      g = random(200,256);
      b = random(0,50);
      
      /*Red LED random integer ranges
      r = random(200,256);
      g = random(0,256);
      b = random(0,50);*/
      displayColor(r,g,b);
    }
    else{
      delay(noteDuration); //delay for the duration of the rest
    }
    // to distinguish the notes, set a minimum time (the note's duration + 30%) between them.
    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
    // stop the current tone from playing to prepare for the next note/song:
    noTone(SPEAKER_PIN);
  }
}
